#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include "fire.h"

int main(int argc,char **argv) {
    struct user_args args;
    struct forest *burning_forest;
    struct forest *next_burning_forest;
    struct drand48_data rand_buffer;

    args = get_user_args(argc,argv);

    // Fork off some threads to execute the following section. All of the
    // variables declared above will have private copies for the threads so
    // that each thread can run a separate simulation of the fire with a
    // different value for burn ratio.
#pragma omp parallel private(args,burning_forest,next_burning_forest,\
        rand_buffer)
    {
        // Divide the 0..1 interval evenly among the threads.  Each thread
        // gets a unique burn ratio and will run a simulation with it.
        args.burn_ratio = (float)omp_get_thread_num()/(omp_get_num_threads()-1);

        srand48_r((time(NULL)+getpid()), &rand_buffer);

        if((burning_forest = init_forest(args,&rand_buffer)) == NULL) {
            fprintf(stderr,"Forest initialization failed\n");
            abort();
        }
        if((next_burning_forest = malloc_forest(args,&rand_buffer)) == NULL) {
            fprintf(stderr,"Next forest initialization failed\n");
            abort();
        }

        if((burn_trees(args,burning_forest,next_burning_forest,&rand_buffer)) < 0) {
            fprintf(stderr,"Burning forest failed\n");
            exit(EXIT_FAILURE);
        }
    }

    free(next_burning_forest);
    free(burning_forest);
    exit(EXIT_SUCCESS);
}

